local SorhaFrameDrag = LibStub("AceAddon-3.0"):GetAddon("SorhaFrameDrag")
local L = LibStub("AceLocale-3.0"):GetLocale("SorhaFrameDrag")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local MODNAME = "ObjectiveTracker"
local ObjectiveTracker = SorhaFrameDrag:NewModule(MODNAME, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0");






--Locals
local ignoreSetPoint = false;
local questTrackerInit = false;
local lazySetPosition = false;
local frameName = "ObjectiveTrackerFrame"

--Defaults
local db;
local dbCore;
local defaults = {
	profile = {
		Dragable = false,
		ForceLocation = false,
		Location = {X = 100, Y = 100, Point = "TOPLEFT", RelativePoint = "TOPLEFT"},
		Height = 500,
	}
}

--Options
local options
local function getOptions()
	if not options then
		options = {
			name = L["Objective Tracker Settings"],
			type = "group",
			childGroups = "tab",
			order = 2,
			arg = MODNAME,
			args = {
				Main = {
					name = "Main",
					type = "group",
					order = 1,
					args = {
						enabled = {
							order = 1,
							type = "toggle",
							name = L["Enable Module"],
							get = function() return SorhaFrameDrag:GetModuleEnabled(MODNAME) end,
							set = function(info, value) 
								SorhaFrameDrag:SetModuleEnabled(MODNAME, value) 
							end,
						},
						DragableEnabled = {
							type = "toggle",
							name = L["Make Dragable"],
							width = "full",
							get = function() return db.Dragable end,
							set = function(info, value) 
								db.Dragable = value;
								if (value == true) then
									ObjectiveTracker:EnableDragging();
								else
									ObjectiveTracker:DisableDragging();
								end
							end,
							order = 2,
						},
						ForceLocationdEnabled = {
							type = "toggle",
							name = L["Force Position"],
							width = "full",
							get = function() return db.ForceLocation end,
							set = function(info, value) 
								db.ForceLocation = value;
								if (value == true) then
									ObjectiveTracker:SetPosition();
								end
							end,
							order = 3,
						},
						Height = {
							name = L["Objective Tracker Height"],
							desc = L["Set the height of the Objective Tracker"],
							type = 'range',
							min = 140,
							max = math.floor(GetScreenHeight()),
							step = 1, bigStep = 5,
							get = function() return db.Height end,
							set = function(_, value)
								db.Height = value
								ObjectiveTrackerFrame:SetHeight(value)
							end,
							order = 4,
						},
					},
				},
			}
		}
	end

	return options
end




--Hooks

local function hook_ObjectiveTrackerFrameSetPoint()
	if (dbCore.DebugEnabled == true) then
		print ('hook_ObjectiveTrackerFrameSetPoint Called was mine:' .. tostring(ignoreSetPoint))
		--print(debugstack())
	end
	if (ignoreSetPoint == true or db.ForceLocation == false or ObjectiveTracker:IsEnabled() == false) then
		return;
	end
	
	ObjectiveTracker:SetPosition();	
	ignoreSetPoint = false;
end

--Inits
function ObjectiveTracker:OnInitialize()
	self.db = SorhaFrameDrag.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile
	dbCore = SorhaFrameDrag.db.profile
	self:SetEnabledState(SorhaFrameDrag:GetModuleEnabled(MODNAME))
	SorhaFrameDrag:RegisterModuleOptions(MODNAME, getOptions, L["Objective Tracker"])
end

function ObjectiveTracker:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")

	if (db.Dragable == true) then
		self:EnableDragging();
	end
end

function ObjectiveTracker:OnDisable()
	self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	self:DisableDragging();
end

function ObjectiveTracker:Refresh()
	db = self.db.profile
	dbCore = SorhaFrameDrag.db.profile
end


--Events
function ObjectiveTracker:PLAYER_ENTERING_WORLD(...)
	if (db.ForceLocation == true) then
		self:SetPosition();
	end
end

function ObjectiveTracker:PLAYER_REGEN_ENABLED(...)
	if (lazySetPosition == true) then
		lazySetPosition = false;
		self:SetPosition();
	end

	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
end

--Quest Tracker
function ObjectiveTracker:InitDragableObjectiveTracker()
	local frame = _G[frameName];
	if (frame ~= nil) then
		self:SecureHook(ObjectiveTrackerFrame, "SetPoint", hook_ObjectiveTrackerFrameSetPoint)
		frame:EnableMouse(1)
		frame:SetClampedToScreen(true)
		frame:RegisterForDrag("LeftButton")
		
		frame:SetScript("OnDragStart", function(self)
			if (self:IsMovable() == true) then
				frame:StartMoving();
			end
		end)
		frame:SetScript("OnDragStop",  function(self)
			self:StopMovingOrSizing()
			if (db.ForceLocation == true) then
				local strPoint, tempB, strRelativePoint, intPosX, intPosY = self:GetPoint();
				db.Location.Point = strPoint
				db.Location.RelativePoint = strRelativePoint
				db.Location.X = intPosX
				db.Location.Y = intPosY				
				ObjectiveTracker:SetPosition();
			end
		end)
		frame:SetMovable(true) --Force blizzard to ignore positioning
		frame:SetUserPlaced(true);
		frame:SetMovable(false)
		questTrackerInit = true;
	end
end

function ObjectiveTracker:EnableDragging()
	if (questTrackerInit == false) then
		self:InitDragableObjectiveTracker();
	end
	local frame = _G[frameName];
	if (frame) then
		frame:SetMovable(true)
	end
end

function ObjectiveTracker:DisableDragging()
	if (questTrackerInit == true) then
		local frame = _G[frameName];
		if (frame) then
			frame:SetMovable(false)
		end
	end
end

function ObjectiveTracker:SetPosition()
	if (db.ForceLocation == false) then
		return;
	end

	if (InCombatLockdown()) then
		lazySetPosition = true;
		self:RegisterEvent("PLAYER_REGEN_ENABLED");
		return
	end
	if (questTrackerInit == false) then
		self:InitDragableObjectiveTracker();
	end

	local frame = _G[frameName];
	if (frame ~= nil) then
		ignoreSetPoint = true;
		frame:SetHeight(db.Height)
		frame:ClearAllPoints()
		
		frame:SetPoint(db.Location.Point, UIParent, db.Location.RelativePoint, db.Location.X, db.Location.Y);
		ignoreSetPoint = false;
	end
end
